@@ -1,36 +1,9 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by Anton Berezin  <tobez@plab.ku.dk>
-#
-#  $Id$
-
 package main;
 
 package Prima::Application;
 use strict;
+use warnings;
 use Prima::Classes;
 use vars qw($uses);
 
@@ -664,7 +637,9 @@ of C<sv::XXX> constants. The constants are:
 	sv::ColorPointer     - 1 if system accepts color pointer icons.
 	sv::CanUTF8_Input    - 1 if system can generate key codes in unicode 
 	sv::CanUTF8_Output   - 1 if system can output utf8 text
-	sv::CompositeDisplay - 1 if system uses double-buffering for the desktop
+	sv::CompositeDisplay - 1 if system uses double-buffering and alpha composition for the desktop, 
+	                       0 if it doesn't, -1 if unknown
+	sv::LayeredWidgets   - 1 if system supports layering
 
 The method can be called with a class string instead of an object instance.
 
@@ -695,6 +670,19 @@ latter case, the loop can be safely re-started.
 Effectively blocks the graphic output for all widgets.
 The output can be restored with C<unlock()>.
 
+=item load_font FONTNAME
+
+Registers font resource in system-specific format. The resource is
+freed after prgram ends.
+
+Notes for win32: To add a font whose information comes from several
+resource files, point FONTNAME to a string with the file names separated by a
+C<|> - for example, C< abcxxxxx.pfm | abcxxxxx.pfb >.
+
+Notes for unix: available only when Prima is compiled with fontconfig and Xft .
+
+Returns number of font resources added.
+
 =item open_help TOPIC
 
 Opens the help viewer window with TOPIC string in 
