@@ -1,32 +1,3 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dk@plab.ku.dk>
-#
-#  $Id$
-
 # combo styles
 package 
     cs;
@@ -35,6 +6,7 @@ use constant DropDown     =>  1;
 use constant DropDownList =>  2;
 
 use strict;
+use warnings;
 
 package Prima::ComboBox;
 
@@ -173,7 +145,7 @@ sub init
 	if ( $self->{autoHeight} && $self->{style} != cs::Simple) {
 		$self->check_auto_size;
 		( $w, $h) = ( $self-> size);
-		$eh = $self-> height;
+		$eh = $self-> geomHeight;
 		$self->{edit}->set( height => $eh, bottom => $h - $eh );
 	}
 
@@ -524,7 +496,7 @@ sub InputLine_Change
 
 	if ( $style == cs::DropDownList) {
 		$edit-> {interaction} = 1;
-		$edit-> text( $list-> get_item_text( $matchId));
+		$edit-> text( $list-> get_item_text( $matchId) // '');
 		$edit-> {interaction} = undef;
 	}
 	$list-> {interaction} = undef;
