@@ -1,37 +1,9 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by Dmitry Karasik <dk@plab.ku.dk>
 #  Modifications by Anton Berezin <tobez@tobez.org>
-#
-#  $Id$
-
-
 package Prima::DetailedList;
 
 use strict;
+use warnings;
 use Prima::Classes;
 use Prima::Lists;
 use Prima::Header;
@@ -96,12 +68,10 @@ sub init
 	my $bw = $self-> borderWidth;
 	my @sz = $self-> size;
 
-	my $hasv = $self-> {vScroll};
-
 	$self-> {header} = $self-> insert( $profile{headerClass} =>
 		name     => 'Header',
 		origin   => [ $bw, $sz[1] - $bw - $hh],
-		size     => [ $sz[0] - $bw * 2 + 1 - ( $hasv ? $self-> {vScrollBar}-> width : 0), $hh],
+		size     => [ $sz[0] - $bw * 2 + 1, $hh],
 		vertical => 0,
 		growMode => gm::Ceiling,
 		items    => $profile{headers},
