@@ -1,36 +1,9 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by Dmitry Karasik <dk@plab.ku.dk>
 #  Modifications by Anton Berezin <tobez@tobez.org>
-#
-#  $Id$
-
 package Prima::Header;
 
 use strict;
+use warnings;
 use Prima::Classes;
 
 use vars qw(@ISA);
@@ -99,6 +72,12 @@ sub on_paint
 	( $self-> disabledColor, $self-> disabledBackColor);
 	my @c3d  = ( $self-> light3DColor, $self-> dark3DColor);
 
+	my ($prelightPart, $prelightColor) = (-1);
+	if ( defined $self->{prelight} ) {
+		$prelightColor = $self-> prelight_color($c[1]);
+		$prelightPart = $self->{prelight};
+	}
+
 	$self-> rect3d( 0, 0, $size[0]-1, $size[1]-1, 1, @c3d, $c[1]);
 	my $v = $self-> {vertical};
 	my ( $x, $y) = ( - $self-> {offset}, ( $size[1] - $self-> {fontHeight}) / 2);
@@ -120,6 +99,10 @@ sub on_paint
 		$v ?
 			$self-> clipRect( 1, $d < 1 ? 1 : $d, $size[0] - 2, $mx) :
 			$self-> clipRect( $d < 1 ? 1 : $d, 1, $mx, $size[1] - 2);
+		if ( $i == $prelightPart) {
+			$self-> color($prelightColor);
+			$self-> bar(0,0,@size);
+		}
 		$self-> color( $c[0]);
 		$v ?
 			$notifier-> ( @notifyParms, $canvas, $i, 1, $d + 1, $size[0] - 2, $mx - 1, $d + 4) :
@@ -295,16 +278,25 @@ sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
 	unless ( $self-> {transaction}) {
-		my $p = $self-> point2area( $x, $y);
-		my $ptr;
-		if ( defined $p && $p < 0) {
-			$ptr = $self-> {vertical} ? cr::SizeNS : cr::SizeWE;
-		} elsif ( $self-> {dragable} && !$self-> {clickable} && defined $p) {
-			$ptr = cr::Move;
-		} else {
-			$ptr = cr::Default;
+		if ( $self-> enabled ) {
+			my $p = $self-> point2area( $x, $y);
+			my $ptr;
+			if ( defined $p && $p < 0) {
+				$ptr = $self-> {vertical} ? cr::SizeNS : cr::SizeWE;
+			} elsif ( $self-> {dragable} && !$self-> {clickable} && defined $p) {
+				$ptr = cr::Move;
+			} else {
+				$ptr = cr::Default;
+			}
+			$self-> pointer( $ptr);
+
+			my $prelight = (defined($p) && $p >= 0) ? $p : undef;
+			if (( $prelight // -1 ) != ( $self->{prelight} // -1)) {
+				$self->{prelight} = $prelight;
+				$self->repaint;
+			}
+
 		}
-		$self-> pointer( $ptr);
 		return;
 	}
 
@@ -373,6 +365,12 @@ sub on_mousemove
 	}
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	$self-> repaint if defined( delete $self->{prelight} );
+}
+
 sub on_mouseclick
 {
 	$_[0]-> clear_event;
