@@ -37,6 +37,9 @@
 #ifdef HAVE_X11_EXTENSIONS_XRANDR_H
 #include <X11/extensions/Xrandr.h>
 #endif
+#ifdef HAVE_X11_EXTENSIONS_XCOMPOSITE_H
+#include <X11/extensions/Xcomposite.h>
+#endif
 #undef Font
 #undef Drawable
 #undef Bool
@@ -121,6 +124,7 @@ typedef struct _PrimaXImage
 #define CACHE_LOW_RES        3
 #define CACHE_LAYERED        4
 #define CACHE_LAYERED_ALPHA  5
+#define CACHE_A8             6
 
 typedef struct {
    int type;
@@ -599,6 +603,9 @@ typedef struct _UnixGuts
    Bool                         xft_no_antialias;
    Bool				randr_extension;
    Bool				render_extension;
+   Bool				composite_extension;
+   int				composite_opcode;
+   Bool				composite_error_triggered;
    struct MsgDlg               *message_boxes;
    XWindow                      grab_redirect;
    Handle                       grab_widget;
@@ -770,6 +777,7 @@ typedef struct _drawable_sys_data
       unsigned sizemax_set              : 1;
       unsigned sync_paint               : 1;
       unsigned task_listed              : 1;
+      unsigned title_utf8               : 1;
       unsigned transparent              : 1;
       unsigned transparent_busy         : 1;
       unsigned want_visible             : 1;
@@ -790,6 +798,10 @@ typedef struct _drawable_sys_data
    uint32_t * xft_map8;
    double     xft_font_cos;
    double     xft_font_sin;
+   XftDraw  * xft_shadow_drawable;
+   Point      xft_shadow_extentions;
+   Pixmap     xft_shadow_pixmap;
+   GC         xft_shadow_gc;
 #endif
 #ifdef HAVE_X11_EXTENSIONS_XRENDER_H
    Picture    argb_picture;
@@ -840,7 +852,9 @@ typedef struct _menu_sys_data
    MenuWindow           wstatic;
    PCachedFont          font;
    int                  guillemots;
+   Bool                 layered;
    unsigned long        c[ciMaxId+1];
+   unsigned long        argb_c[ciMaxId+1];
    Color                rgb[ciMaxId+1];
    XWindow              focus;
    PMenuWindow          focused;
@@ -1223,6 +1237,9 @@ prima_xft_init( void);
 extern void
 prima_xft_done( void);
 
+extern void
+prima_xft_gp_destroy( Handle self );
+
 extern Bool
 prima_xft_font_pick( Handle self, Font * source, Font * dest, double * size, XftFont ** xft_result);
 
@@ -1282,3 +1299,24 @@ prima_gtk_done( void);
 extern char *
 prima_gtk_openfile( char * params);
 #endif
+
+typedef struct _ViewProfile {
+  Point        pos;
+  Point        size;
+  Bool         visible;
+  Bool         focused;
+  Handle       capture;
+  char *       title;
+  int          shape_count;
+  int          shape_ordering;
+  XRectangle * shape_rects;
+} ViewProfile, *PViewProfile;
+
+extern void
+prima_set_view_ex( Handle self, PViewProfile p);
+
+extern void
+prima_get_view_ex( Handle self, PViewProfile p);
+
+extern int
+prima_flush_events( Display * disp, XEvent * ev, Handle self);
