@@ -19,8 +19,8 @@ DeviceBitmap_init( Handle self, HV * profile)
    inherited init( self, profile);
    var-> w = pget_i( width);
    var-> h = pget_i( height);
-   var-> monochrome = pget_B( monochrome);
-   if ( !apc_dbm_create( self, var-> monochrome))
+   var-> type = pget_i( type);
+   if ( !apc_dbm_create( self, var-> type))
       croak("Cannot create device bitmap");
    inherited begin_paint( self);
    opt_set( optInDraw);
@@ -38,39 +38,48 @@ Bool DeviceBitmap_begin_paint      ( Handle self) { return true;}
 Bool DeviceBitmap_begin_paint_info ( Handle self) { return true;}
 void DeviceBitmap_end_paint        ( Handle self) { return;}
 
-Bool
-DeviceBitmap_monochrome( Handle self, Bool set, Bool monochrome)
+int
+DeviceBitmap_type( Handle self, Bool set, int type)
 {
    if ( set)
-      croak("Attempt to write read-only property %s", "DeviceBitmap::monochrome");
-   return var-> monochrome;
+      croak("Attempt to write read-only property %s", "DeviceBitmap::type");
+   return var-> type;
 }
 
-static Handle xdup( Handle self, char * className)
+static Handle xdup( Handle self, Bool icon)
 {
    Handle h;
    PDrawable i;
    HV * profile = newHV();
    Point s;
+   int rop;
 
    pset_H( owner,        var-> owner);
    pset_i( width,        var-> w);
    pset_i( height,       var-> h);
-   pset_i( type,         var-> monochrome ? imMono : imRGB);
+   if ( icon && var-> type == dbtLayered) {
+      pset_i( type,      imRGB);
+      pset_i( maskType,  imbpp8);
+      pset_i( autoMasking, 0);
+      rop = ropSrcCopy;
+   } else {
+      pset_i( type,      (var-> type == dbtBitmap) ? imBW : imRGB);
+      rop = ropCopyPut;
+   }
 
-   h = Object_create( className, profile);
+   h = Object_create( icon ? "Prima::Icon" : "Prima::Image", profile);
    sv_free(( SV *) profile);
    i = ( PDrawable) h;
    s = i-> self-> get_size( h);
    i-> self-> begin_paint( h);
-   i-> self-> put_image_indirect( h, self, 0, 0, 0, 0, s.x, s.y, s.x, s.y, ropCopyPut);
+   i-> self-> put_image_indirect( h, self, 0, 0, 0, 0, s.x, s.y, s.x, s.y, rop);
    i-> self-> end_paint( h);
    --SvREFCNT( SvRV( i-> mate));
    return h;
 }
 
-Handle DeviceBitmap_image( Handle self) { return xdup( self, "Prima::Image"); }
-Handle DeviceBitmap_icon( Handle self) { return xdup( self, "Prima::Icon"); }
+Handle DeviceBitmap_image( Handle self) { return xdup( self, false); }
+Handle DeviceBitmap_icon( Handle self) { return xdup( self, true); }
 
 SV *
 DeviceBitmap_get_handle( Handle self)
