@@ -318,7 +318,7 @@ Drawable_get_physical_palette( Handle self)
 }
 
 SV *
-Drawable_get_font_abc( Handle self, int first, int last, Bool unicode)
+Drawable_get_font_abcdef( Handle self, int first, int last, Bool unicode, PFontABC (*func)(Handle, int, int, Bool))
 {
    int i;
    AV * av;
@@ -336,7 +336,7 @@ Drawable_get_font_abc( Handle self, int first, int last, Bool unicode)
    else {
      gpARGS;
      gpENTER( newRV_noinc(( SV *) newAV()));
-     abc = apc_gp_get_font_abc( self, first, last, unicode );
+     abc = func( self, first, last, unicode );
      gpLEAVE;
    }
 
@@ -352,6 +352,18 @@ Drawable_get_font_abc( Handle self, int first, int last, Bool unicode)
    return newRV_noinc(( SV *) av);
 }
 
+SV *
+Drawable_get_font_abc( Handle self, int first, int last, Bool unicode)
+{
+   return Drawable_get_font_abcdef( self, first, last, unicode, apc_gp_get_font_abc);
+}
+
+SV *
+Drawable_get_font_def( Handle self, int first, int last, Bool unicode)
+{
+   return Drawable_get_font_abcdef( self, first, last, unicode, apc_gp_get_font_def);
+}
+
 SV *
 Drawable_get_font_ranges( Handle self)
 {
@@ -430,12 +442,17 @@ Bool
 Drawable_text_out( Handle self, SV * text, int x, int y)
 {
    Bool ok;
-   STRLEN dlen;
-   char * c_text = SvPV( text, dlen);
-   Bool   utf8 = prima_is_utf8_sv( text);
-   if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
-   ok = apc_gp_text_out( self, c_text, x, y, dlen, utf8);
-   if ( !ok) perl_error();
+   if ( !SvROK( text )) {
+      STRLEN dlen;
+      char * c_text = SvPV( text, dlen);
+      Bool   utf8 = prima_is_utf8_sv( text);
+      if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
+      ok = apc_gp_text_out( self, c_text, x, y, dlen, utf8);
+      if ( !ok) perl_error();
+   } else {
+      SV * ret = sv_call_perl(text, "text_out", "<Hii", self, x, y);
+      ok = ret && SvTRUE(ret);
+   }
    return ok;
 }
 
@@ -875,13 +892,21 @@ Drawable_get_text_width( Handle self, SV * text, Bool addOverhang)
 {
    gpARGS;
    int res;
-   STRLEN dlen;
-   char * c_text = SvPV( text, dlen);
-   Bool   utf8 = prima_is_utf8_sv( text);
-   if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
-   gpENTER(0);
-   res = apc_gp_get_text_width( self, c_text, dlen, addOverhang, utf8);
-   gpLEAVE;
+   if ( !SvROK( text )) {
+      STRLEN dlen;
+      char * c_text = SvPV( text, dlen);
+      Bool   utf8 = prima_is_utf8_sv( text);
+      if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
+      gpENTER(0);
+      res = apc_gp_get_text_width( self, c_text, dlen, addOverhang, utf8);
+      gpLEAVE;
+   } else {
+      SV * ret;
+      gpENTER(0);
+      ret = sv_call_perl(text, "get_text_width", "<Hi", self, addOverhang);
+      gpLEAVE;
+      res = (ret && SvOK(ret)) ? SvIV(ret) : 0;
+   }
    return res;
 }
 
@@ -892,23 +917,31 @@ Drawable_get_text_box( Handle self, SV * text)
    Point * p;
    AV * av;
    int i;
-   STRLEN dlen;
-   char * c_text = SvPV( text, dlen);
-   Bool   utf8 = prima_is_utf8_sv( text);
-   if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
-   gpENTER( newRV_noinc(( SV *) newAV()));
-   p = apc_gp_get_text_box( self, c_text, dlen, utf8);
-   gpLEAVE;
+   if ( !SvROK( text )) {
+      STRLEN dlen;
+      char * c_text = SvPV( text, dlen);
+      Bool   utf8 = prima_is_utf8_sv( text);
+      if ( utf8) dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
+      gpENTER( newRV_noinc(( SV *) newAV()));
+      p = apc_gp_get_text_box( self, c_text, dlen, utf8);
+      gpLEAVE;
 
-   av = newAV();
-   if ( p) {
-      for ( i = 0; i < 5; i++) {
-         av_push( av, newSViv( p[ i]. x));
-         av_push( av, newSViv( p[ i]. y));
-      };
-      free( p);
+      av = newAV();
+      if ( p) {
+         for ( i = 0; i < 5; i++) {
+            av_push( av, newSViv( p[ i]. x));
+            av_push( av, newSViv( p[ i]. y));
+         };
+         free( p);
+      }
+      return newRV_noinc(( SV *) av);
+   } else {
+      SV * ret;
+      gpENTER( newRV_noinc(( SV *) newAV()));
+      ret = newSVsv(sv_call_perl(text, "get_text_box", "<H", self));
+      gpLEAVE;
+      return ret;
    }
-   return newRV_noinc(( SV *) av);
 }
 
 static PFontABC
@@ -1246,12 +1279,22 @@ Drawable_do_text_wrap( Handle self, TextWrapRec * t)
 SV*
 Drawable_text_wrap( Handle self, SV * text, int width, int options, int tabIndent)
 {
+   gpARGS;
    TextWrapRec t;
    Bool retChunks;
    char** c;
    int i;
    AV * av;
    STRLEN tlen;
+   int returnFirstLine = ( t. options & twReturnFirstLineLength) == twReturnFirstLineLength;
+
+   if ( SvROK( text )) {
+      SV * ret;
+      gpENTER( returnFirstLine ? newSViv(0) : newRV_noinc(( SV *) newAV()));
+      ret = newSVsv(sv_call_perl(text, "text_wrap", "<Hiii", self, width, options, tabIndent));
+      gpLEAVE;
+      return ret;
+   }
 
    t. text      = SvPV( text, tlen);
    t. utf8_text = prima_is_utf8_sv( text);
@@ -1269,7 +1312,9 @@ Drawable_text_wrap( Handle self, SV * text, int width, int options, int tabInden
    t. unicode   = &var-> font_abc_unicode;
    t. t_char    = nil;
 
+   gpENTER( returnFirstLine ? newSViv(0) : newRV_noinc(( SV *) newAV()));
    c = Drawable_do_text_wrap( self, &t);
+   gpLEAVE;
 
    if (( t. options & twReturnFirstLineLength) == twReturnFirstLineLength) {
       IV rlen = 0;
