@@ -83,6 +83,7 @@ void Window_update_sys_handle( Handle self, HV * profile)
        pexist( taskListed) ||
        pexist( borderIcons) ||
        pexist( onTop) ||
+       pexist( layered) ||
        pexist( borderStyle)
     )) return;
    if ( pexist( owner)) my-> cancel_children( self);
@@ -95,7 +96,8 @@ void Window_update_sys_handle( Handle self, HV * profile)
       pexist( windowState) ? pget_i( windowState) : my-> get_windowState( self),
       pexist( onTop) ? pget_B( onTop) : -1, /* This is way better. I should've thought of this before! */
       !( pexist( originDontCare) && pget_B( originDontCare)),
-      !( pexist( sizeDontCare)   && pget_B( sizeDontCare))
+      !( pexist( sizeDontCare)   && pget_B( sizeDontCare)),
+      pexist( layered)     ? pget_B( layered) : my-> get_layered( self)
    ))
       croak("Cannot create window");
    pdelete( borderStyle);
@@ -104,6 +106,7 @@ void Window_update_sys_handle( Handle self, HV * profile)
    pdelete( taskListed);
    pdelete( windowState);
    pdelete( onTop);
+   pdelete( layered);
 }
 
 void Window_handle_event( Handle self, PEvent event)
@@ -399,12 +402,6 @@ Window_execute_shared( Handle self, Handle insertBefore)
    return apc_window_execute_shared( self, insertBefore);
 }
 
-Bool
-Window_layered( Handle self, Bool set, Bool layered)
-{
-   return false;
-}
-
 Bool
 Window_modalHorizon( Handle self, Bool set, Bool modalHorizon)
 {
@@ -449,8 +446,6 @@ void Window_set( Handle self, HV * profile)
    dPROFILE;
    Bool owner_icon = false;
    
-   pdelete( layered);
-   
    if ( pexist( menuFont)) {
       SvHV_Font( pget_sv( menuFont), &Font_buffer, "Window::set");
       my-> set_menu_font( self, Font_buffer);
