@@ -234,7 +234,7 @@ set_motif_hints( XWindow window, int border_style, int border_icons)
 Bool
 apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
 		   int border_style, Bool task_list, int window_state, 
-		   int on_top, Bool use_origin, Bool use_size)
+		   int on_top, Bool use_origin, Bool use_size, Bool layered)
 {
    DEFXX;
    Handle real_owner;
@@ -245,8 +245,10 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
    XWMHints wmhints;
    XClassHint *class_hint;
    ConfigureEventPair *cep;
-   Bool layered = false;
    unsigned long valuemask;
+   Bool recreate;
+   ViewProfile vprf;
+   XWindow old = X_WINDOW;
    
    if ( border_style != bsSizeable) border_style = bsDialog;
    border_icons &= biAll;
@@ -254,6 +256,43 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
    if ( !guts. argb_visual. visual || guts. argb_visual. visualid == guts. visual. visualid)
       layered = false;
 
+   recreate = X_WINDOW ? (layered != XX->flags.layered) : false;
+   if ( recreate ) {
+      int i, count;
+      Handle * list;
+      XEvent dummy_ev;
+
+      list  = PWidget(self)-> widgets. items;
+      count = PWidget(self)-> widgets. count;
+      CWidget(self)-> end_paint_info( self);
+      CWidget(self)-> end_paint( self);
+      prima_release_gc( XX);
+      for( i = 0; i < count; i++)
+         prima_get_view_ex( list[ i], ( ViewProfile*)( X( list[ i])-> recreateData = malloc( sizeof( ViewProfile))));
+      
+      if ( XX-> recreateData) {
+         memcpy( &vprf, XX-> recreateData, sizeof( vprf));
+         free( XX-> recreateData);
+         XX-> recreateData = nil;
+      } else
+         prima_get_view_ex( self, &vprf);
+      if ( guts. currentMenu && PComponent( guts. currentMenu)-> owner == self) prima_end_menu();
+      apc_window_set_menu( self, nilHandle);
+      CWidget( self)-> end_paint_info( self);
+      CWidget( self)-> end_paint( self);
+      if ( XX-> flags. paint_pending) {
+         TAILQ_REMOVE( &guts.paintq, XX, paintq_link);
+         XX-> flags. paint_pending = false;
+      }
+      /* flush configure events */
+      XSync( DISP, false);
+      while ( XCheckIfEvent( DISP, &dummy_ev, (XIfEventProcType)prima_flush_events, (XPointer)self));
+      hash_delete( guts.windows, (void*)&old, sizeof(old), false);
+      hash_delete( guts.windows, (void*)&(XX->client), sizeof(XX->client), false);
+      X_WINDOW = 0;
+      XCHECKPOINT;
+   }
+
    if ( X_WINDOW) { /* recreate request */
       Bool destructive_motif_hints = 0; /* KDE 3.1: setting motif hints kills net_wm hints */
       if ( !guts.icccm_only && (
@@ -448,13 +487,38 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
    XX-> flags. clip_owner = false;
    XX-> flags. sync_paint = sync_paint;
    XX-> flags. task_listed = 1;
+   XX-> flags. layered = XX-> flags. layered_requested = layered;
 
    XX-> above = nilHandle;
    XX-> owner = real_owner;
-   apc_component_fullname_changed_notify( self);
-   prima_send_create_event( X_WINDOW);
+
    if ( on_top > 0) NETWM_SET_ON_TOP( X_WINDOW, 1);
    apc_window_task_listed( self, task_list);
+
+   if (recreate) {
+      int i;
+      int  count = PWidget(self)->widgets. count;
+      Handle * list = PWidget(self)->widgets. items;
+      Point pos;
+
+      pos = PWidget(self)-> pos;
+      apc_window_set_menu( self, PWindow( self)-> menu);
+      bzero( &hints, sizeof( XSizeHints));
+      hints. flags  = PBaseSize;
+      hints. width  = hints. base_width  = XX-> size. x;
+      hints. height = hints. base_height = XX-> size. y;
+      XSetWMNormalHints( DISP, X_WINDOW, &hints);
+      prima_set_view_ex( self, &vprf);
+      XX-> ackOrigin = pos;
+      XX-> ackSize   = XX-> size;
+      XX-> flags. mapped = XX-> flags. want_visible;
+      for ( i = 0; i < count; i++) ((( PComponent) list[ i])-> self)-> recreate( list[ i]);
+      XDestroyWindow( DISP, old);
+      return true;
+   }
+
+   apc_component_fullname_changed_notify( self);
+   prima_send_create_event( X_WINDOW);
    if ( border_style == bsSizeable) XX-> flags. sizeable = 1;
 
    /* setting initial size */
@@ -690,6 +754,7 @@ apc_window_set_caption( Handle self, const char *caption, Bool utf8)
          PropModeReplace, ( unsigned char*) caption, strlen( caption) + 1);
       XChangeProperty( DISP, X_WINDOW, NET_WM_ICON_NAME, UTF8_STRING, 8, 
          PropModeReplace, ( unsigned char*) caption, strlen( caption) + 1);
+      X(self)->flags. title_utf8 = 1;
    } else {
       XDeleteProperty( DISP, X_WINDOW, NET_WM_NAME);
       XDeleteProperty( DISP, X_WINDOW, NET_WM_ICON_NAME);
@@ -698,6 +763,7 @@ apc_window_set_caption( Handle self, const char *caption, Bool utf8)
          XSetWMName( DISP, X_WINDOW, &p);
          XFree( p. value);
       }
+      X(self)->flags. title_utf8 = 0;
    }
    XFlush( DISP);
    return true;
@@ -777,8 +843,7 @@ apc_SetWMNormalHints( Handle self, XSizeHints * hints)
    }
    XSetWMNormalHints( DISP, X_WINDOW, hints);
    XCHECKPOINT;
-}   
-
+}
 
 Bool
 apc_window_set_client_pos( Handle self, int x, int y)
